<?php

class RabbitLoader_21_Public{

    static $skip_reason = '';

    public static function addActions(){
        //is_user_logged_in is a pluggable function and you could get a fatal error if you call it too early.
        add_action('init', 'RabbitLoader_21_Public::init', -1);
        add_action('shutdown', 'RabbitLoader_21_Public::shutdown', 10000);
        add_filter('wp_redirect', 'RabbitLoader_21_Public::wp_redirect', 10, 2);
        add_filter('redirect_canonical', 'RabbitLoader_21_Public::redirect_canonical', 10, 2);
        add_filter('pre_handle_404','RabbitLoader_21_Public::pre_handle_404', 10, 2);
        RabbitLoader_21_CanonicalUrl::init();
        RabbitLoader_21_Util_WP::init();
        
        //Third party theme hooks
        add_action('kirki_output_inline_styles', function($value){
            #theme developed using kirki framework may generate lots of inline css referring to font files making the page load slow
            #https://github.com/kirki-framework/kirki/blob/3a5d5093b1a99d1c7b813608479fe8fc9348b6a3/packages/kirki-framework/module-css/src/CSS.php
            $value = false;
            return $value;
        }, -1);
    }

    public static function init()
    {  
        if(!empty($_SERVER["HTTP_X_RL_PINGBACK_V2"])){
            self::setup_endpoint();
            return;
        }

        try{
            if(!empty($_SERVER["HTTP_X_RL_PINGBACK"])){
                $responses = ['time'=>date('c')];
                $responses['saved_count'] = self::pull_optimized_copy($isAPIError, $apiMessage, $cur_id);
                $responses['is_api_error'] = $isAPIError;
                $responses['api_message'] = $apiMessage;
                $responses['cur_id'] = $cur_id;

                RabbitLoader_21_Core::clean_orphaned_cached_files(200);
                RabbitLoader_21_Core::sendJsonResponse($responses);
            }else if(!empty($_SERVER["HTTP_X_RL_SET_OPTION"]) && strcmp($_SERVER["HTTP_X_RL_SET_OPTION"], 'rabbitloader_field_disconnect_reason')===0){
                $responses = ['time'=>date('c')];
                try{
                    global $wpdb;
                    $responses['disconnect_reason'] = $wpdb->get_results("select option_id, option_name from $wpdb->options where option_name in ('rabbitloader_field_disconnect_reason', 'rabbitloader_field_update_time')");
                }catch(Throwable $e){
                    RabbitLoader_21_Core::on_exception($e);
                }
                RabbitLoader_21_Core::sendJsonResponse($responses);
            }else if(!empty($_SERVER["HTTP_X_RL_APIKEY"])){

                $responses = ['time'=>date('c')];
                if($_SERVER["HTTP_X_RL_APIKEY"] == get_option('rabbitloader_field_apikey')){

                    if(!empty($_SERVER["HTTP_X_RL_DIAGNOSIS"])){
                        $constants = get_defined_constants(true);
                        $constants = empty($constants['user']) ? [] : $constants['user'];
                        //remove known sensitive info
                        $sensitive_constants = ['DB_HOST', 'DB_NAME', 'DB_USER', 'DB_PASSWORD', 'AUTH_KEY', 'SECURE_AUTH_KEY', 'LOGGED_IN_KEY', 'NONCE_KEY', 'AUTH_SALT', 'SECURE_AUTH_SALT', 'LOGGED_IN_SALT', 'NONCE_SALT', 'COOKIEHASH', 'USER_COOKIE', 'PASS_COOKIE', 'AUTH_COOKIE', 'SECURE_AUTH_COOKIE', 'LOGGED_IN_COOKIE', 'RECOVERY_MODE_COOKIE'];
                        foreach($sensitive_constants as $const_name){
                            unset($constants[$const_name]);
                        }
                        $responses['server'] = $_SERVER;
                        $responses['constants'] = $constants;
                        $responses['classes'] = get_declared_classes();
                        
                        try{
                            global $wpdb;
                            $responses['options'] = $wpdb->get_results("select option_id, option_name from $wpdb->options");
                        }catch(Throwable $e){
                            RabbitLoader_21_Core::on_exception($e);
                        }
                    }
                    if(!empty($_SERVER["HTTP_X_RL_PURGE"])){
                        RabbitLoader_21_Core::purge_all($purge_count, 'RL_PANEL');
                        $responses['purged'] = 1;
                        $responses['purge_count'] = $purge_count;
                    }
                    if(!empty($_SERVER["HTTP_X_RL_SET_OPTION"])){
                        $responses['option'] = get_option($_SERVER["HTTP_X_RL_SET_OPTION"]);
                    }
                    if(!empty($_SERVER["HTTP_X_RL_CACHE_MISSED_LOG"])){
                        RabbitLoader_21_Core::get_log_file('cache_missed', $cache_missed);
                        if(file_exists($cache_missed)){
                            $responses['cache_missed'] = @file_get_contents($cache_missed);
                        }else{
                            $responses['cache_missed'] = 'FILE_NOT_EXISTS';
                        }
                    }
                    if(isset($_SERVER["HTTP_X_RL_WARMUP"])){
                        $queued_offset = intval($_SERVER["HTTP_X_RL_WARMUP"]);
                        RabbitLoader_21_Core::push_recent_posts($queued_offset, $queued_count, $published_count);
                        $responses['queued_offset'] = $queued_offset;
                        $responses['queued_count'] = $queued_count;
                        $responses['published_count'] = $published_count;
                        RabbitLoader_21_Core::clean_orphaned_cached_files(1);
                    }
                }else{
                    //plugin might be disconnected
                    $responses['error'] = 'key_mismatch';
                }
                RabbitLoader_21_Core::sendJsonResponse($responses);
            }
        }catch(Throwable $e){
            RabbitLoader_21_Core::on_exception($e);
        }

        if (current_user_can( 'manage_options' )) {
            add_action('admin_bar_menu', 'RabbitLoader_21_Public::adminBarMenu', 50);
            add_action('wp_enqueue_scripts', 'RabbitLoader_21_Public::adminBarScript');
        }
    }

    public static function shutdown($cache_served = false)
    {
        try{
            if(!$cache_served){
                self::optimized_version_missing();
                RabbitLoader_21_Core::clean_orphaned_cached_files(300);
            }else{
                RabbitLoader_21_Core::clean_orphaned_cached_files(3600);
            }
        }catch(Throwable $e){
            RabbitLoader_21_Core::on_exception($e);
        }
    }

    private static function can_cache_request($request_uri){
        if(!empty(self::$skip_reason)){
            return false;
        }
        if(RabbitLoader_21_Util_Core::get_request_type()!='get'){
            //Not a GET request, return
            self::$skip_reason = 'method-'.RabbitLoader_21_Util_Core::get_request_type();
            return false;
        }

        if(RabbitLoader_21_Util_WP::is_user_logged_in() || RabbitLoader_21_Util_WP::is_login_page()){
            //we are not serving cached content to logged in users
            self::$skip_reason = 'user-session';
            return false;
        }

        if(RabbitLoader_21_Util_WP::is_ajax()){
            self::$skip_reason = 'ajax';
            return false;
        }
        
        if(!empty($_SERVER["HTTP_X_RL_PINGBACK"]) || !empty($_SERVER['HTTP_RL_NO_OPTIMIZATION']) || !empty($_GET['rl-no-optimization']) || !empty($_SERVER["HTTP_X_RL_DIAGNOSIS"])){
            //its origin pull form ur own server, let it get the original HTML
            self::$skip_reason = 'rl-origin';
            return false;
        }

        if(!empty($_COOKIE['woocommerce_items_in_cart'])){
            //TBD- user has some items in cart, in future version we will check how to serve a cached page with cart
            self::$skip_reason = 'cart-items';
            return false;
        }

        if(empty($_SERVER["HTTP_HOST"]) || RabbitLoader_21_Util_WP::is_cli() || !empty($_GET["mwprid"])){
            self::$skip_reason = 'cli';
            return false;
        }

        global $wp_query;
        if (isset($wp_query) && $wp_query->is_404()) {
            self::$skip_reason = '404-not-found';
            return false;
        }

        if (RabbitLoader_21_Util_WP::is_search()) {
            self::$skip_reason = 'search';
            return false;
        }

        RabbitLoader_21_Core::getWpUserOption($user_options);
        if(!empty($user_options['exclude_patterns'])){
            $exclude_patterns = explode("\n", $user_options['exclude_patterns']);
            if(!empty($exclude_patterns)){
                $matched = false;
                foreach($exclude_patterns as $path_pattern){
                    $matched = fnmatch(trim($path_pattern), $request_uri);
                    if($matched){break;}
                }
            }
            if($matched){
                self::$skip_reason = 'exclude-pattern';
                return false;
            }
        }

        if(!empty(self::$skip_reason)){
            self::$skip_reason = self::$skip_reason;
            return false;
        }

        return true;
    }

    public static function process_incoming_request($mode='ac'){
        try{
            RabbitLoader_21_Util_Core::get_requested_uri($request_uri, $request_url);
            
            if(!self::can_cache_request($request_uri)){
                if(empty(self::$skip_reason)){self::$skip_reason='unknown';}
                RabbitLoader_21_Core::sendHeader('x-rl-cache: skipped-'.self::$skip_reason, true);
                return;
            }

            if(!empty($request_uri) && $request_uri!="/favicon.ico"){

                RabbitLoader_21_Core::get_cache_file_path($request_url, $cache_file);

                if(self::serve_from_cache_if_available($cache_file)){
                    //ignore buffer and serve from cache
                    self::shutdown(true);
                    exit;
                }

                RabbitLoader_21_Core::sendHeader('x-rl-cache: miss/'.$mode, true);
            }
        }catch(Throwable $e){
            RabbitLoader_21_Core::on_exception($e);
        }
    }

    private static function optimized_version_missing(){

        RabbitLoader_21_Util_Core::get_requested_uri($request_uri, $request_url);
        if(self::can_cache_request($request_uri)){
            $request_url.="\n";
            RabbitLoader_21_Util_Core::fac('cache_missed', $request_url, WP_DEBUG);
        }else{
            //error_log($skip_reason.'=>'.$request_uri);
        }
    }

    private static function pull_optimized_copy(&$isAPIError, &$apiMessage, &$cur_id){

        $saved_count = 0;
        $cur_id = '';
        
        RabbitLoader_21_Util_Core::get_requested_uri($request_uri, $request_url);
        RabbitLoader_21_Core::get_cache_file_path($request_url, $cache_file);
        RabbitLoader_21_TP::purge_url($request_url, $tp_purge_count);
        
        $http = RabbitLoader_21_Core::callGETAPI('page/html?uri='.urlencode($request_uri).'&url='.urlencode($request_url), $isAPIError, $apiMessage);

        if(!empty($http['response']['code']) && intval($http['response']['code'])==200){
            if(!empty($http['body']['data']['content']) && $http['body']['data']['response_code']==200){

                $headers = json_encode($http['body']['data']['headers']);
                //RabbitLoader_21_Util_Core::send_headers($headers);
                //echo $http['body']['data']['content'];
                if(RabbitLoader_21_Util_Core::fpc($cache_file.'_h', $headers, WP_DEBUG)){
                    ++$saved_count;
                }else{
                    $apiMessage = 'Local file saving failed at '.$cache_file.'_h';
                }
                if(RabbitLoader_21_Util_Core::fpc($cache_file.'_c', $http['body']['data']['content'], WP_DEBUG)){
                    ++$saved_count;
                }else{
                    $apiMessage = 'Local file saving failed at '.$cache_file.'_c';
                }

                if(!empty($http['body']['data']['cur_id'])){
                    $cur_id = $http['body']['data']['cur_id'];
                }

                RabbitLoader_21_TP::purge_url($request_url, $tp_purge_count);
            }
        }

        return $saved_count;
    }

    private static function setup_endpoint(){
        if(empty($_SERVER["HTTP_X_RL_APIKEY"]) || empty($_POST['canonical_url'])){
            return;
        }

        $response = [
            'time'=>date('c')
        ];
        $saved_count = 0;
        $tp_purge_count = 0;
        RabbitLoader_21_Core::getWpOption($rl_wp_options);
        if((!empty($rl_wp_options['rabbitloader_field_apikey']) && ($rl_wp_options['rabbitloader_field_apikey'] == $_SERVER["HTTP_X_RL_APIKEY"])) || (function_exists('get_option') && !empty($_SERVER["HTTP_X_RL_APIKEY"]) && ($_SERVER["HTTP_X_RL_APIKEY"] == get_option('rabbitloader_field_apikey')))){
            
            $cur_id = RabbitLoader_21_Util_Core::get_param('cur_id', true);
            $response['cur_id'] = $cur_id;

            try{
                $zip = new \ZipArchive();
                if (!empty($cur_id) && !empty($_FILES["content_zip"]["tmp_name"]) && $zip->open($_FILES["content_zip"]["tmp_name"]) === TRUE) {
                    $dir = sys_get_temp_dir();
                    $zip->extractTo($dir);
                    $zip->close();
                    RabbitLoader_21_Core::get_cache_file_path($_POST['canonical_url'], $cache_file);

                    $html_fn = "$dir/$cur_id.html";
                    if(file_exists($html_fn)){
                        $response['zip_opened'] = true;

                        if(RabbitLoader_21_Util_Core::move($html_fn, $cache_file.'_c')){
                            ++$saved_count;
                        }
                        if(RabbitLoader_21_Util_Core::move("$dir/$cur_id.json", $cache_file.'_h')){
                            ++$saved_count;
                        }

                        RabbitLoader_21_TP::purge_url($_POST['canonical_url'], $tp_purge_count);

                    }else{
                        $response['zip_extract'] = "$html_fn not found";
                    }

                    $response['saved_count'] = $saved_count;
                    $response['tp_purge_count'] = $tp_purge_count;
                    $response['cache_file'] = $cache_file;
                    $response['canonical_url'] = $_POST['canonical_url'];
                }else{
                    $response['zip_opened'] = false;
                }
            }catch(\Throwable $e){
                $response['exception'] = $e->getMessage();
            }
        }else{
            $response['exception'] = 'key mismatch';
        }
        http_response_code(200);
        echo json_encode($response);
        exit;
    }
    private static function serve_from_cache_if_available($cache_file){
        if(RabbitLoader_21_Core::cache_exists_for_hash($cache_file, 0)){
            //ignore buffer and serve from cache
            if(file_exists($cache_file.'_h')){
                RabbitLoader_21_Util_Core::send_headers(file_get_contents($cache_file.'_h'));
            }
            RabbitLoader_21_Core::sendHeader('x-rl-cache: hit', true);
            echo file_get_contents($cache_file.'_c');
            return true;
        }
    }

    public static function adminBarMenu($admin_bar){
        $top_menu = array(
            'id' => 'rabbitloader_top_menu',
            'title' => 'RabbitLoader',
            'href' => admin_url( 'options-general.php?page=rabbitloader' )
        );

        $list_menu = array(
            'parent' => 'rabbitloader_top_menu',
            'id'     => 'rabbitloader_purge_page',
            'title'  =>  'Purge cache for this page',
            'href'   =>  "#",
            'meta' => array(
                'class' => 'rabbitloader_purge_page',
            )
        );

        $admin_bar->add_node($top_menu);
        $admin_bar->add_node($list_menu);
    }

    public static function adminBarScript(){
        global $post;

        wp_enqueue_script('rabbitloader-index', RABBITLOADER_PLUG_URL . 'admin/js/index.js', ['jquery'], RABBITLOADER_PLUG_VERSION);
        wp_localize_script( 'rabbitloader-index', 'rabbitloader_local_vars', [
            'admin_ajax' => admin_url( 'admin-ajax.php' ),
            'post_id' => empty($post) ? 0: $post->ID
        ]);
    }

    public static function wp_redirect($location, $status){
        if(!empty($location)){
            self::$skip_reason = 'redirect-'.$status;
        }
        return $location;
    }

    public static function redirect_canonical($redirect_url, $requested_url){
        if(!empty($redirect_url)){self::$skip_reason = 'redirect-canonical';}
        return $redirect_url;
    }

    public static function pre_handle_404($preempt, $wp_query ){
        if($preempt){self::$skip_reason = '404';}
        return $preempt;
    }
}

?>